@@ -1,8 +1,6 @@
 package Munin::Master::UpdateWorker;
 use base qw(Munin::Master::Worker);
 
-# $Id$
-
 use warnings;
 use strict;
 
@@ -58,8 +56,20 @@ sub do_work {
     my $path = $self->{host}->get_full_path;
     $path =~ s{[:;]}{-}g;
 
-    my $nodedesignation = $host."/".
-	$self->{host}{address}.":".$self->{host}{port};
+    # Parameters are space-separated from the main address
+    my ($url, $params) = split(/ +/, $self->{host}{address}, 2);
+    my $uri = new URI($url);
+
+    # If the scheme is not defined, it's a plain host.
+    # Prefix it with munin:// to be able to parse it like others
+    $uri = new URI("munin://" . $url) unless $uri->scheme;
+
+    my $nodedesignation;
+    if ($uri->scheme eq "ssh" || $uri->scheme eq "cmd") {
+      $nodedesignation = $host." (".$self->{host}{address}.")";
+    }else{
+      $nodedesignation = $host." (".$self->{host}{address}.":".$self->{host}{port}.")";
+    }
 
     my $lock_file = sprintf ('%s/munin-%s.lock',
 			     $config->{rundir},
@@ -241,7 +251,7 @@ sub do_work {
 	}
 
 	if ($EVAL_ERROR =~ m/^NO_SPOOLFETCH_DATA /) {
-	    INFO "[INFO] No spoofetch data for $nodedesignation";
+	    INFO "[INFO] No spoolfetch data for $nodedesignation";
 	    return;
 	} elsif ($EVAL_ERROR) {
 	    ERROR "[ERROR] Error in node communication with $nodedesignation: "
@@ -616,7 +626,7 @@ sub _update_rrd_files {
 	    DEBUG "[DEBUG] asking for a rrd of size : " . $ds_config->{graph_data_size};
 
 	    # Avoid autovivification (for multigraphs)
-	    my $first_epoch = (defined($service_data) and defined($service_data->{$ds_name})) ? ($service_data->{$ds_name}->{when}->[0]) : 0;
+	    my $first_epoch = (defined($service_data) and defined($service_data->{$ds_name}) and $service_data->{$ds_name}->{when}) ? ($service_data->{$ds_name}->{when}->[0]) : 0;
 	    my $rrd_file = $self->_create_rrd_file_if_needed($service, $ds_name, $ds_config, $first_epoch);
 
 	    if (defined($service_data) and defined($service_data->{$ds_name})) {
